const std = @import("std");
const Clipboard = @import("clipboard.zig");

pub fn main() !u8 {
    var watch = false;
    var gpa = std.heap.GeneralPurposeAllocator(.{}){};
    defer _ = gpa.deinit();
    const allocator = gpa.allocator();
    var args = std.process.args();
    defer args.deinit();
    while (args.next(allocator)) |arg_or_err| {
        const arg = try arg_or_err;
        defer allocator.free(arg);
        if (std.mem.eql(u8, arg, "-w")) {
            watch = true;
            break;
        }
    }
    var clip = try Clipboard.init(allocator);
    defer clip.deinit();
    if (watch) {
        while (clip.download()) |text| {
            std.log.debug("Got remote clipboard: {s}", .{text});
            addToClipboardNoError(allocator, text);
            allocator.free(text);
        }
    } else {
        if (clip.download()) |text| {
            std.log.debug("Got remote clipboard: {s}", .{text});
            addToClipboardNoError(allocator, text);
            allocator.free(text);
        }
    }

    return 0;
}

fn addToClipboardNoError(allocator: std.mem.Allocator, data: []const u8) void {
    addToClipboard(allocator, data) catch |e| {
        std.log.err("Could not add data to clipboard: {}", .{e});
        if (@errorReturnTrace()) |trace| {
            std.debug.dumpStackTrace(trace.*);
        }
    };
}

fn addToClipboard(allocator: std.mem.Allocator, data: []const u8) !void {
    // We're going to cheat here and just run xclip
    const xclip_cmd = try std.fmt.allocPrint(allocator, "echo '{s}' | xclip -selection c", .{data});
    defer allocator.free(xclip_cmd);

    const result = try std.ChildProcess.exec(.{
        .allocator = allocator,
        .argv = &[_][]const u8{
            "/usr/bin/env",
            "sh",
            "-c",
            xclip_cmd,
        },
    });
    try std.io.getStdErr().writer().writeAll(result.stderr);
    switch (result.term) {
        .Exited => |code| if (code != 0) return error.NonZeroExit,
        .Signal => return error.FailedWithSignal,
        .Stopped => return error.WasStopped,
        .Unknown => return error.Failed,
    }
}
